@@ -2356,6 +2356,7 @@ FvwmWindow *AddWindow(
 	{
 		XEvent e;
 
+		memset(&e, 0, sizeof(e));
 		FWarpPointer(
 			dpy, Scr.Root, Scr.Root, 0, 0, Scr.MyDisplayWidth,
 			Scr.MyDisplayHeight,
@@ -2363,6 +2364,7 @@ FvwmWindow *AddWindow(
 			fw->frame_g.y + (fw->frame_g.height>>1));
 		e.xany.type = ButtonPress;
 		e.xbutton.button = 1;
+		e.xbutton.state = Button1Mask;
 		e.xbutton.x_root = fw->frame_g.x + (fw->frame_g.width>>1);
 		e.xbutton.y_root = fw->frame_g.y + (fw->frame_g.height>>1);
 		e.xbutton.x = (fw->frame_g.width>>1);
@@ -2472,6 +2474,7 @@ FvwmWindow *AddWindow(
 void FetchWmProtocols(FvwmWindow *tmp)
 {
 	Atom *protocols = NULL, *ap;
+	long *l_protocols;
 	int i, n;
 	Atom atype;
 	int aformat;
@@ -2510,10 +2513,11 @@ void FetchWmProtocols(FvwmWindow *tmp)
 			     dpy, FW_W(tmp), _XA_WM_PROTOCOLS, 0L, 10L, False,
 			     _XA_WM_PROTOCOLS, &atype, &aformat, &nitems,
 			     &bytes_remain,
-			     (unsigned char **)&protocols)) == Success)
+			     (unsigned char **)&l_protocols)) == Success)
 		{
-			for (i = 0, ap = protocols; i < nitems; i++, ap++)
+			for (i = 0; i < nitems; i++)
 			{
+				ap = &(l_protocols[i]);
 				if (*ap == (Atom)_XA_WM_TAKE_FOCUS)
 				{
 					SET_WM_TAKES_FOCUS(tmp, 1);
@@ -2791,15 +2795,10 @@ void GetWindowSizeHints(FvwmWindow *fw)
 				"The applicaton window (window id %#lx)\n"
 				"  \"%s\" has broken aspect ratio: "
 				"%d/%d - %d/%d\n"
-				"    fvwm is ignoring this aspect ratio.  "
-				"If you are having a problem with the\n"
-				"    application, send a bug report with this "
-				"message included to the\n"
-				"    application owner.  "
-				"There is no need to notify "
-				"fvwm-workers@fvwm.org.\n",
+				"    fvwm is ignoring this aspect ratio.  ",
 				FW_W(fw), fw->name.name, minAspectX,
 				minAspectY, maxAspectX, maxAspectY);
+			fvwm_msg_report_app();
 		}
 		else
 		{
@@ -2846,11 +2845,6 @@ void GetWindowSizeHints(FvwmWindow *fw)
 			"The application window (id %#lx)\n"
 			"  \"%s\" has broken size hints (%s).\n"
 			"    fvwm is ignoring those hints.  "
-			"If you are having a problem with the\n"
-			"    application, send a bug report with this "
-			"message included to the\n"
-			"    application owner.  "
-			"There is no need to notify fvwm-workers@fvwm.org.\n"
 			"  hint override = %d, flags = %lx\n"
 			"  min_width = %d, min_height = %d, "
 			"max_width = %d, max_height = %d\n"
@@ -2868,6 +2862,7 @@ void GetWindowSizeHints(FvwmWindow *fw)
 			orig_hints.max_aspect.x, orig_hints.max_aspect.y,
 			orig_hints.base_width, orig_hints.base_height,
 			orig_hints.win_gravity);
+		fvwm_msg_report_app();
 	}
 
 	return;
@@ -3026,8 +3021,8 @@ void destroy_window(FvwmWindow *fw)
 		}
 		adjust_fvwm_internal_windows(fw);
 		BroadcastPacket(
-			M_DESTROY_WINDOW, 3, FW_W(fw), FW_W_FRAME(fw),
-			(unsigned long)fw);
+			M_DESTROY_WINDOW, 3, (long)FW_W(fw),
+			(long)FW_W_FRAME(fw), (unsigned long)fw);
 		EWMH_DestroyWindow(fw);
 		focus_grab_buttons_on_layer(fw->layer);
 		Scr.FWScheduledForDestroy = flist_append_obj(
@@ -3044,8 +3039,9 @@ void destroy_window(FvwmWindow *fw)
 	{
 		SET_SCHEDULED_FOR_DESTROY(fw, 1);
 		adjust_fvwm_internal_windows(fw);
-		BroadcastPacket(M_DESTROY_WINDOW, 3,
-				FW_W(fw), FW_W_FRAME(fw), (unsigned long)fw);
+		BroadcastPacket(
+			M_DESTROY_WINDOW, 3, (long)FW_W(fw),
+			(long)FW_W_FRAME(fw), (unsigned long)fw);
 		EWMH_DestroyWindow(fw);
 	}
 	focus_grab_buttons_on_layer(fw->layer);
