@@ -243,11 +243,12 @@ static int ParseBinding(
 	char key_string[201] = "", buffer[80], *windowName = NULL, *p;
 	int button = 0;
 	int n1=0,n2=0,n3=0;
-	KeySym keysym = NoSymbol;
 	int context;
 	int modifier;
-	Bool is_unbind_request = False, is_pass_thru = False;
 	int rc;
+	KeySym keysym = NoSymbol;
+	Bool is_unbind_request = False;
+	Bool is_pass_through = False;
 	Bool is_binding_removed = False;
 	Binding *b;
 	Binding *rmlist = NULL;
@@ -257,7 +258,13 @@ static int ParseBinding(
 
 	/* tline points after the key word "Mouse" or "Key" */
 	token = p = PeekToken(tline, &ptr);
-	/* check to see if a window name has been specified. */
+ 	/* check to see if a window name has been specified. */
+	if (p == NULL)
+	{
+		fvwm_msg(
+			ERR, "ParseBinding", "empty %s binding, ignored\n",tline);
+		return 0;
+	}
 	if (*p == '(')
 	{
 		/* A window name has been specified for the binding. */
@@ -268,8 +275,12 @@ static int ParseBinding(
 			if (*p == '\0')
 			{
 				if (!is_silent)
-					fvwm_msg(ERR, "ParseBinding",
-						"Syntax error in line %s - missing ')'", tline);
+				{
+					fvwm_msg(
+						ERR, "ParseBinding",
+						"Syntax error in line %s -"
+						" missing ')'", tline);
+				}
 				return 0;
 			}
 			++p;
@@ -279,8 +290,12 @@ static int ParseBinding(
 		if (*p != '\0')
 		{
 			if (!is_silent)
-				fvwm_msg(ERR, "ParseBinding",
-					"Syntax error in line %s - trailing text after specified window", tline);
+			{
+				fvwm_msg(
+					ERR, "ParseBinding",
+					"Syntax error in line %s - trailing"
+					" text after specified window", tline);
+			}
 			return 0;
 		}
 		token = PeekToken(ptr, &ptr);
@@ -454,8 +469,8 @@ static int ParseBinding(
 
 	if (action)
 	{
-		is_pass_thru = actionIsPassThru(action);
-		if (is_pass_thru)
+		is_pass_through = is_pass_through_action(action);
+		if (is_pass_through)
 		{
 			/* pass-through actions indicate that the event be
 			 * allowed to pass through to the underlying window. */
@@ -472,7 +487,7 @@ static int ParseBinding(
 		}
 	}
 	/* see if it is an unbind request */
-	if (!action || (action[0] == '-' && !is_pass_thru))
+	if (!action || (action[0] == '-' && !is_pass_through))
 	{
 		is_unbind_request = True;
 	}
