@@ -47,6 +47,7 @@
 #include "stack.h"
 #include "commands.h"
 #include "decorations.h"
+#include "virtual.h"
 
 /* ---------------------------- local definitions -------------------------- */
 
@@ -151,12 +152,12 @@ static FvwmWindow *Circulate(
 				return found;
 			}
 		}
-		if (!fw || fw == &Scr.FvwmRoot)
+		if (fw == NULL || fw == &Scr.FvwmRoot)
 		{
 			if (Direction > 0)
 			{
 				/* Go to end of list */
-				for (fw = &Scr.FvwmRoot; fw && fw->next;
+				for (fw = Scr.FvwmRoot.next; fw && fw->next;
 				     fw = fw->next)
 				{
 					/* nop */
@@ -516,31 +517,36 @@ void CreateConditionMask(char *flags, WindowConditionMask *mask)
 		}
 		else if (StrEquals(cond,"FixedSize"))
 		{
-		        /* don't set mask here, because we make the test here
+			/* don't set mask here, because we make the test here
 			   (and don't compare against window's mask)
 			   by checking allowed function */
-		        SET_SIZE_FIXED(mask, on);
+			SET_SIZE_FIXED(mask, on);
 			SETM_SIZE_FIXED(mask, 1);
 		}
+		else if (StrEquals(cond, "FixedPosition"))
+		{
+			SET_FIXED(mask, on);
+			SETM_FIXED(mask, 1);
+		}
 		else if (StrEquals(cond,"HasHandles"))
 		{
-		        SET_HAS_HANDLES(mask,on);
-			SETM_HAS_HANDLES(mask,1);
+			SET_HAS_HANDLES(mask, on);
+			SETM_HAS_HANDLES(mask, 1);
 		}
 		else if (StrEquals(cond,"Iconifiable"))
 		{
-		        SET_IS_UNICONIFIABLE(mask,!on);
-		        SETM_IS_UNICONIFIABLE(mask,1);
+			SET_IS_UNICONIFIABLE(mask, !on);
+			SETM_IS_UNICONIFIABLE(mask, 1);
 		}
 		else if (StrEquals(cond,"Maximizable"))
 		{
-		        SET_IS_UNMAXIMIZABLE(mask,!on);
-		        SETM_IS_UNMAXIMIZABLE(mask,1);
+			SET_IS_UNMAXIMIZABLE(mask, !on);
+			SETM_IS_UNMAXIMIZABLE(mask, 1);
 		}
 		else if (StrEquals(cond,"Closable"))
 		{
-		        SET_IS_UNCLOSABLE(mask,!on);
-		        SETM_IS_UNCLOSABLE(mask,1);
+			SET_IS_UNCLOSABLE(mask, !on);
+			SETM_IS_UNCLOSABLE(mask, 1);
 		}
 		else if (StrEquals(cond,"Shaded"))
 		{
@@ -716,8 +722,21 @@ Bool MatchesConditionMask(FvwmWindow *fw, WindowConditionMask *mask)
 	    !is_function_allowed(F_RESIZE,NULL,fw,True,False))
 	{
 	        return False;
-        }
-        SETM_SIZE_FIXED(mask, 0);
+	}
+	SETM_SIZE_FIXED(mask, 0);
+	if (IS_FIXED(mask) && 
+	    mask->flag_mask.common.s.is_fixed &&
+	    is_function_allowed(F_MOVE, NULL, fw, True, False))
+	{       
+	        return False;
+	}
+	if (!IS_FIXED(mask) && 
+	    mask->flag_mask.common.s.is_fixed &&
+	    !is_function_allowed(F_MOVE, NULL, fw, True, False))
+	{       
+	        return False;
+	}
+	SETM_FIXED(mask, 0);
 	if (IS_UNICONIFIABLE(mask) &&
 	    mask->flag_mask.common.s.is_uniconifiable &&
 	    is_function_allowed(F_ICONIFY,NULL,fw,True,False))
@@ -1034,6 +1053,17 @@ static void direction_cmd(F_CMD_ARGS, Bool is_scan)
 	{
 		cycle = True;
 		tmp = PeekToken(action, &action);
+		if ( ! tmp )
+		{
+			fvwm_msg(
+				ERR, "Direction", "Missing minor direction %s",
+				(tmp) ? tmp : "");
+			if (cond_rc != NULL)
+			{
+				cond_rc->rc = COND_RC_ERROR;
+			}
+			return;
+		}
 		dir2 = gravity_parse_dir_argument(tmp, NULL, -1);
 		/* if enum direction_t changes, this is trashed. */
 		if (dir2 == -1 || dir2 > DIR_NW ||
@@ -1730,15 +1760,15 @@ void CMD_Test(F_CMD_ARGS)
 	char *restofline;
 	char *flags;
 	char *condition;
-	char *tmp;
+	char *flags_ptr;
 	int match;
 	int error;
 
 	flags = CreateFlagString(action, &restofline);
 
 	/* Next parse the flags in the string. */
-	tmp = flags;
-	tmp = GetNextSimpleOption(tmp, &condition);
+	flags_ptr = flags;
+	flags_ptr = GetNextSimpleOption(flags_ptr, &condition);
 
 	match = 1;
 	error = 0;
@@ -1765,11 +1795,12 @@ void CMD_Test(F_CMD_ARGS)
 		else if (StrEquals(cond, "Version"))
 		{
 			char *pattern;
-			tmp = GetNextSimpleOption(tmp, &pattern);
+			flags_ptr = GetNextSimpleOption(flags_ptr, &pattern);
 			if (pattern)
 			{
 				char *ver;
-				tmp = GetNextSimpleOption(tmp, &ver);
+				flags_ptr = GetNextSimpleOption(
+					flags_ptr, &ver);
 				if (ver == NULL)
 				{
 					match = matchWildcards(
@@ -1848,7 +1879,7 @@ void CMD_Test(F_CMD_ARGS)
 				/* cannot happen */
 				break;
 			}
-			tmp = GetNextSimpleOption(tmp, &pattern);
+			flags_ptr = GetNextSimpleOption(flags_ptr, &pattern);
 			if (pattern)
 			{
 				match = cond_check_access(pattern, type, im);
@@ -1859,6 +1890,160 @@ void CMD_Test(F_CMD_ARGS)
 				error = 1;
 			}
 		}
+		else if (StrEquals(cond, "EnvIsSet"))
+		{
+			char *var_name;
+			flags_ptr = GetNextSimpleOption(flags_ptr, &var_name);
+			if (var_name)
+			{
+				const char *value = getenv(var_name);
+				match = value != NULL;
+			}
+			else
+			{
+				error = 1;
+			}
+		}
+		else if (StrEquals(cond, "EnvMatch"))
+		{
+			char *var_name;
+			flags_ptr = GetNextSimpleOption(flags_ptr, &var_name);
+			if (var_name)
+			{
+				const char *value = getenv(var_name);
+				char *pattern;
+				/* unfortunately, GetNextSimpleOption is
+				 * broken, does not accept quoted empty "" */
+				flags_ptr = GetNextSimpleOption(
+					flags_ptr, &pattern);
+				if (!value)
+				{
+					value = "";
+				}
+				if (pattern)
+				{
+					match =
+						/* include empty string case */
+						(!pattern[0] && !value[0])
+						||
+						matchWildcards(pattern, value);
+				}
+				else
+				{
+					error = 1;
+				}
+			}
+			else
+			{
+				error = 1;
+			}
+		}
+		else if (StrEquals(cond, "EdgeIsActive"))
+		{
+			direction_t dir= DIR_NONE;
+			char *dirname;
+			char *next;
+			next = GetNextSimpleOption(flags_ptr, &dirname);
+			if (dirname)
+			{
+				dir = gravity_parse_dir_argument(
+					dirname, NULL, DIR_NONE);
+				if (dir == DIR_NONE)
+				{
+					if (!StrEquals(dirname, "Any"))
+					{
+						next = flags_ptr;
+					}
+				}
+				else if (dir > DIR_MAJOR_MASK)
+				{
+					error = 1;
+				}
+				free(dirname);
+			}
+
+			if (!error)
+			{
+				if (((dir == DIR_W || dir == DIR_NONE) &&
+					Scr.PanFrameLeft.isMapped) ||
+				    ((dir == DIR_N || dir == DIR_NONE) &&
+					Scr.PanFrameTop.isMapped) ||
+				    ((dir == DIR_S || dir == DIR_NONE) &&
+					Scr.PanFrameBottom.isMapped) ||
+				    ((dir == DIR_E || dir == DIR_NONE) &&
+ 				    	Scr.PanFrameRight.isMapped))
+				{
+				  	match = 1;
+				}
+				else
+				{
+				  	match = 0;
+				}
+			}
+			flags_ptr = next;
+		}
+		else if (StrEquals(cond, "EdgeHasPointer"))
+		{
+			int x,y;
+			Window win;
+			direction_t dir = DIR_NONE;
+			char *dirname;
+			char *next;
+			next = GetNextSimpleOption(flags_ptr, &dirname);
+			if (dirname)
+			{
+				dir = gravity_parse_dir_argument(
+					dirname, NULL, DIR_NONE);
+				if (dir == DIR_NONE)
+				{
+					if (!StrEquals(dirname, "Any"))
+					{
+						next = flags_ptr;
+					}
+				}
+				else if (dir > DIR_MAJOR_MASK)
+				{
+					error = 1;
+				}
+				free(dirname);
+			}
+
+			if (!error)
+			{
+				if (FQueryPointer(
+					dpy, Scr.Root, &JunkRoot, &win,
+					&JunkX, &JunkY,	&x, &y, &JunkMask)
+ 					== False)
+				{
+					/* pointer is on a different screen */
+					match = 0;
+				}
+				else if (is_pan_frame(win))
+				{
+					if (dir == DIR_NONE ||
+					    (dir == DIR_N &&
+					     win == Scr.PanFrameTop.win) ||
+					    (dir == DIR_S &&
+					     win == Scr.PanFrameBottom.win) ||
+					    (dir == DIR_E &&
+					     win == Scr.PanFrameRight.win) ||
+					    (dir == DIR_W &&
+					     win == Scr.PanFrameLeft.win))
+					{
+						match = 1;
+					}
+					else
+					{
+						match = 0;
+					}
+				}
+				else
+				{
+					match = 0;
+				}
+			}
+			flags_ptr = next;
+		}
 		else
 		{
 			/* unrecognized condition */
@@ -1874,8 +2059,9 @@ void CMD_Test(F_CMD_ARGS)
 		{
 			break;
 		}
-		tmp = GetNextSimpleOption(tmp, &condition);
+		flags_ptr = GetNextSimpleOption(flags_ptr, &condition);
 	}
+
 	if (flags != NULL)
 	{
 		free(flags);
