@@ -28,8 +28,8 @@ void EWMH_SetDesktopViewPort(void);
 void EWMH_SetDesktopGeometry(void);
 
 void EWMH_SetActiveWindow(Window w);
-void EWMH_SetWMDesktop(FvwmWindow *fwin);
-void EWMH_SetWMState(FvwmWindow *fwin, Bool do_restore);
+void EWMH_SetWMDesktop(FvwmWindow *fw);
+void EWMH_SetWMState(FvwmWindow *fw, Bool do_restore);
 
 int EWMH_IsKdeSysTrayWindow(Window w);
 void EWMH_ManageKdeSysTray(Window w, int type);
@@ -37,20 +37,20 @@ void EWMH_SetClientList(void);
 void EWMH_SetClientListStacking(void);
 void EWMH_UpdateWorkArea(void);
 void EWMH_GetWorkAreaIntersection(
-	FvwmWindow *fwin, int *x, int *y, int *h, int *w, int func);
+	FvwmWindow *fw, int *x, int *y, int *h, int *w, int func);
 float EWMH_GetBaseStrutIntersection(
 	int x11, int y11, int x12, int y12, Bool use_percent);
 float EWMH_GetStrutIntersection(
 	int x11, int y11, int x12, int y12, Bool use_percent);
-void EWMH_SetFrameStrut(FvwmWindow *fwin);
-void EWMH_SetAllowedActions(FvwmWindow *fwin);
+void EWMH_SetFrameStrut(FvwmWindow *fw);
+void EWMH_SetAllowedActions(FvwmWindow *fw);
 
-void EWMH_GetIconGeometry(FvwmWindow *fwin, rectangle *icon_rect);
+void EWMH_GetIconGeometry(FvwmWindow *fw, rectangle *icon_rect);
 
-void EWMH_GetStyle(FvwmWindow *fwin, window_style *style);
-void EWMH_WindowInit(FvwmWindow *fwin);
-void EWMH_RestoreInitialStates(FvwmWindow *fwin, int event_type);
-void EWMH_DestroyWindow(FvwmWindow *fwin);
+void EWMH_GetStyle(FvwmWindow *fw, window_style *style);
+void EWMH_WindowInit(FvwmWindow *fw);
+void EWMH_RestoreInitialStates(FvwmWindow *fw, int event_type);
+void EWMH_DestroyWindow(FvwmWindow *fw);
 void EWMH_WindowDestroyed(void);
 
 void EWMH_Init(void);
@@ -63,18 +63,18 @@ Bool EWMH_ProcessClientMessage(const exec_context_t *exc);
 void EWMH_ProcessPropertyNotify(const exec_context_t *exc);
 
 /* ewmh_icon.c */
-void EWMH_DeleteWmIcon(FvwmWindow *fwin, Bool mini_icon, Bool icon);
+void EWMH_DeleteWmIcon(FvwmWindow *fw, Bool mini_icon, Bool icon);
 int EWMH_SetIconFromWMIcon(
-	FvwmWindow *fwin, CARD32 *list, unsigned int size, Bool is_mini_icon);
-void EWMH_DoUpdateWmIcon(FvwmWindow *fwin, Bool mini_icon, Bool icon);
+	FvwmWindow *fw, CARD32 *list, unsigned int size, Bool is_mini_icon);
+void EWMH_DoUpdateWmIcon(FvwmWindow *fw, Bool mini_icon, Bool icon);
 
 /* ewmh_name.c */
-void EWMH_SetVisibleName(FvwmWindow *fwin, Bool is_icon_name);
+void EWMH_SetVisibleName(FvwmWindow *fw, Bool is_icon_name);
 int EWMH_WMName(
-	FvwmWindow *fwin, XEvent *ev, window_style *style, unsigned long any);
+	FvwmWindow *fw, XEvent *ev, window_style *style, unsigned long any);
 int EWMH_WMIconName(
-	FvwmWindow *fwin, XEvent *ev, window_style *style, unsigned long any);
+	FvwmWindow *fw, XEvent *ev, window_style *style, unsigned long any);
 void EWMH_SetDesktopNames(void);
-void EWMH_fullscreen(FvwmWindow *fwin);
+void EWMH_fullscreen(FvwmWindow *fw);
 
 #endif /* _EWMH_ */
