@@ -31,6 +31,7 @@
 #include "config.h"
 
 #include <stdio.h>
+#include <math.h>
 #include <X11/keysym.h>
 
 #include "libs/fvwmlib.h"
@@ -289,29 +290,29 @@ static int GetOnePositionArgument(
 	}
 	else if (sscanf(s1,"w-%d",&val) == 1)
 	{
-		*pFinalX = x - (int)(val*factor);
+		*pFinalX = x - (int)(val*factor + 0.5);
 	}
 	else if (sscanf(s1,"w+%d",&val) == 1 || sscanf(s1,"w%d",&val) == 1 )
 	{
-		*pFinalX = x + (int)(val*factor);
+		*pFinalX = x + (int)(val*factor + 0.5);
 	}
 	else if (sscanf(s1,"m-%d",&val) == 1)
 	{
 		add_pointer_position = True;
-		*pFinalX = -(int)(val*factor);
+		*pFinalX = -(int)(val*factor + 0.5);
 	}
 	else if (sscanf(s1,"m+%d",&val) == 1 || sscanf(s1,"m%d",&val) == 1 )
 	{
 		add_pointer_position = True;
-		*pFinalX = (int)(val*factor);
+		*pFinalX = (int)(val*factor + 0.5);
 	}
 	else if (sscanf(s1,"-%d",&val) == 1)
 	{
-		*pFinalX = max-w - (int)(val*factor);
+		*pFinalX = max-w - (int)(val*factor + 0.5);
 	}
 	else if (sscanf(s1,"+%d",&val) == 1 || sscanf(s1,"%d",&val) == 1)
 	{
-		*pFinalX = (int)(val*factor);
+		*pFinalX = (int)(val*factor + 0.5);
 	}
 	else
 	{
@@ -329,7 +330,10 @@ static int GetOnePositionArgument(
 			/* pointer is on a different screen - that's okay here
 			 */
 		}
-		*pFinalX += (is_x) ? x : y;
+		else
+		{
+			*pFinalX += (is_x) ? x : y;
+		}
 	}
 
 	return 1;
@@ -367,6 +371,7 @@ static int GetMoveArguments(
 	if (s1 && fPointer && StrEquals(s1, "pointer"))
 	{
 		*fPointer = True;
+		*paction = action;
 		free(s1);
 		return 0;
 	}
@@ -485,7 +490,9 @@ static int ParseOneResizeArgument(
 
 static int GetResizeArguments(
 	char **paction, int x, int y, int w_base, int h_base, int w_inc,
-	int h_inc, size_borders *sb, int *pFinalW, int *pFinalH)
+	int h_inc, size_borders *sb, int *pFinalW, int *pFinalH,
+	direction_t *ret_dir, Bool *is_direction_fixed,
+	Bool *do_warp_to_border)
 {
 	int n;
 	char *naction;
@@ -494,7 +501,11 @@ static int GetResizeArguments(
 	char *s2;
 	int w_add;
 	int h_add;
+	int has_frame_option;
 
+	*ret_dir = DIR_NONE;
+	*is_direction_fixed = False;
+	*do_warp_to_border = False;
 	if (!paction)
 	{
 		return 0;
@@ -511,29 +522,70 @@ static int GetResizeArguments(
 
 		n = GetMoveArguments(&naction, 0, 0, &nx, &ny, NULL, NULL);
 		if (n < 2)
+		{
 			return 0;
+		}
 		*pFinalW = nx - x + 1;
 		*pFinalH = ny - y + 1;
 		*paction = naction;
 
 		return n;
 	}
-	if (StrEquals(token, "frame"))
+	has_frame_option = 0;
+	for ( ; ; token = PeekToken(naction, &naction))
+	{
+		if (StrEquals(token, "frame"))
+		{
+			has_frame_option = 1;
+		}
+		else if (StrEquals(token, "direction"))
+		{
+			if (token == NULL)
+			{
+				return 0;
+			}
+			*ret_dir = gravity_parse_dir_argument(
+				naction, &naction, DIR_NONE);
+			if (*ret_dir != DIR_NONE)
+			{
+				*is_direction_fixed = True;
+			}
+		}
+		else if (StrEquals(token, "fixeddirection"))
+		{
+			*is_direction_fixed = True;
+		}
+		else if (StrEquals(token, "warptoborder"))
+		{
+			*do_warp_to_border = True;
+		}
+		else
+		{
+			break;
+		}
+	}
+	if (has_frame_option)
 	{
 		w_add = 0;
 		h_add = 0;
-		naction = GetNextToken(naction, &s1);
 	}
 	else
 	{
 		w_add = sb->total_size.width;
 		h_add = sb->total_size.height;
+	}
+	s1 = NULL;
+		if (token != NULL)
+	{
 		s1 = safestrdup(token);
 	}
 	naction = GetNextToken(naction, &s2);
 	if (!s2)
 	{
-		free(s1);
+		if (s1 != NULL)
+		{
+			free(s1);
+		}
 		return 0;
 	}
 	*paction = naction;
@@ -543,8 +595,14 @@ static int GetResizeArguments(
 		s1, Scr.MyDisplayWidth, w_base, w_inc, w_add, pFinalW);
 	n += ParseOneResizeArgument(
 		s2, Scr.MyDisplayHeight, h_base, h_inc, h_add, pFinalH);
-	free(s1);
-	free(s2);
+	if (s1 != NULL)
+	{
+		free(s1);
+	}
+	if (s2 != NULL)
+	{
+		free(s2);
+	}
 	if (n < 2)
 	{
 		n = 0;
@@ -559,6 +617,8 @@ static int GetResizeMoveArguments(
 	int *pFinalH, Bool *fWarp, Bool *fPointer)
 {
 	char *action = *paction;
+	direction_t dir;
+	Bool dummy;
 
 	if (!paction)
 	{
@@ -566,7 +626,7 @@ static int GetResizeMoveArguments(
 	}
 	if (GetResizeArguments(
 		    &action, *pFinalX, *pFinalY, w_base, h_base, w_inc, h_inc,
-		    sb, pFinalW, pFinalH) < 2)
+		    sb, pFinalW, pFinalH, &dir, &dummy, &dummy) < 2)
 	{
 		return 0;
 	}
@@ -763,7 +823,7 @@ static void DisplaySize(
 	dwidth /= tmp_win->hints.width_inc;
 	dheight /= tmp_win->hints.height_inc;
 
-	(void) sprintf (str, GEOMETRY_WINDOW_SIZE_STRING, dwidth, dheight);
+	(void)sprintf(str, GEOMETRY_WINDOW_SIZE_STRING, dwidth, dheight);
 	if (Init)
 	{
 		XClearWindow(dpy,Scr.SizeWindow);
@@ -1357,17 +1417,20 @@ static void __move_window(F_CMD_ARGS, Bool do_animate, int mode)
 	{
 		rectangle r;
 		rectangle s;
+		rectangle t;
 
 		do_animate = False;
+		r.x = x;
+		r.y = y;
+		r.width = width;
+		r.height = height;
+		get_absolute_geometry(&t, &r);
+		get_page_offset_rectangle(&page_x, &page_y, &t);
 		if (!get_page_arguments(action, &page_x, &page_y))
 		{
 			page_x = Scr.Vx;
 			page_y = Scr.Vy;
 		}
-		r.x = x;
-		r.y = y;
-		r.width = width;
-		r.height = height;
 		s.x = page_x - Scr.Vx;
 		s.y = page_y - Scr.Vy;
 		s.width = Scr.MyDisplayWidth;
@@ -2008,8 +2071,11 @@ Bool __move_loop(
 				KeyPressMask | PointerMotionMask |
 				ButtonMotionMask | ExposureMask, &e)))
 		{
+			XEvent le;
+
+			fev_get_last_event(&le);
 			rc = HandlePaging(
-				&e, dx, dy, &xl, &yt, &delta_x, &delta_y,
+				&le, dx, dy, &xl, &yt, &delta_x, &delta_y,
 				False, False, True);
 			if (rc == 1)
 			{
@@ -2018,68 +2084,88 @@ Bool __move_loop(
 				{
 					x_virtual_offset = 0;
 				}
-				if (XOffset)
-				{
-					xl += XOffset;
-				}
+				xl += XOffset;
 				if (delta_y)
 				{
 					y_virtual_offset = 0;
 				}
-				if (YOffset)
-				{
-					yt += YOffset;
-				}
+				yt += YOffset;
 				if (do_snap)
 				{
 					DoSnapAttract(
-						fw, Width, Height, &xl,
-						&yt);
+						fw, Width, Height, &xl, &yt);
 				}
+				fev_make_null_event(&e, dpy);
 				e.type = MotionNotify;
 				e.xmotion.time = fev_get_evtime();
 				e.xmotion.x_root = xl - XOffset;
 				e.xmotion.y_root = yt - YOffset;
 				e.xmotion.same_screen = True;
-				fev_fake_event(&e);
 				break;
 			}
 		}
 		if (rc == -1)
 		{
 			/* block until an event arrives */
+			/* dv (2004-07-01): With XFree 4.1.0.1, some Mouse
+			 * events are not reported to fvwm when the pointer
+			 * moves very fast and suddenly stops in the corner of
+			 * the screen.  Handle EnterNotify/LeaveNotify events
+			 * too to get an idea where the pointer might be. */
 			FMaskEvent(
 				dpy, ButtonPressMask | ButtonReleaseMask |
 				KeyPressMask | PointerMotionMask |
-				ButtonMotionMask | ExposureMask, &e);
+				ButtonMotionMask | ExposureMask |
+				EnterWindowMask | LeaveWindowMask, &e);
 		}
 
-		/* discard any extra motion events before a logical release */
-		if (e.type == MotionNotify)
+		/* discard extra events before a logical release */
+		if (e.type == MotionNotify ||
+		    e.type == EnterNotify || e.type == LeaveNotify)
 		{
-			XEvent new_event;
-
-			/*** logic borrowed from icewm ***/
 			while (FPending(dpy) > 0 &&
 			       FCheckMaskEvent(
 				       dpy, ButtonMotionMask |
 				       PointerMotionMask | ButtonPressMask |
-				       ButtonRelease | KeyPressMask,
-				       &new_event))
+				       ButtonRelease | KeyPressMask |
+				       EnterWindowMask | LeaveWindowMask, &e))
 			{
-				if (e.type != new_event.type)
+				if (e.type == ButtonPress ||
+				    e.type == ButtonRelease ||
+				    e.type == KeyPress)
 				{
-					FPutBackEvent(dpy, &new_event);
 					break;
 				}
-				else
-				{
-					e = new_event;
-				}
 			}
-			/*** end of code borrowed from icewm ***/
+		}
+		if (e.type == EnterNotify || e.type == LeaveNotify)
+		{
+			XEvent e2;
+			int x;
+			int y;
 
-		} /* if (e.type == MotionNotify) */
+			/* Query the pointer to catch the latest information.
+			 * This *is* necessary. */
+			if (FQueryPointer(
+				    dpy, Scr.Root, &JunkRoot, &JunkChild, &x,
+				    &y, &JunkX, &JunkY, &JunkMask) == True)
+			{
+				fev_make_null_event(&e2, dpy);
+				e2.type = MotionNotify;
+				e2.xmotion.time = fev_get_evtime();
+				e2.xmotion.x_root = x;
+				e2.xmotion.y_root = y;
+				e2.xmotion.state = JunkMask;
+				e2.xmotion.same_screen = True;
+				e = e2;
+				fev_fake_event(&e);
+			}
+			else
+			{
+				/* pointer is on a different screen,
+				 * ignore event */
+			}
+		}
 
 		/* Handle a limited number of key press events to allow
 		 * mouseless operation */
@@ -2161,7 +2247,9 @@ Bool __move_loop(
 				if (button_mask || (e.xbutton.button != 1))
 				{
 					if (!do_move_opaque)
+					{
 						switch_move_resize_grid(False);
+					}
 					if (!IS_ICONIFIED(fw))
 					{
 						*FinalX = fw->frame_g.x;
@@ -2201,8 +2289,7 @@ Bool __move_loop(
 				if (do_snap)
 				{
 					DoSnapAttract(
-						fw, Width, Height, &xl,
-						&yt);
+						fw, Width, Height, &xl, &yt);
 				}
 			}
 
@@ -2217,13 +2304,12 @@ Bool __move_loop(
 			{
 				continue;
 			}
-			if (!(e.xkey.state & Mod1Mask))
+			if (!(e.xmotion.state & Mod1Mask))
 			{
 				nosnap_enabled = True;
 			}
 			do_snap = nosnap_enabled &&
-				(e.xkey.state & Mod1Mask) ? False : True;
-
+				(e.xmotion.state & Mod1Mask) ? False : True;
 			xl = e.xmotion.x_root;
 			yt = e.xmotion.y_root;
 			if (xl > 0 && xl < Scr.MyDisplayWidth - 1)
@@ -2251,18 +2337,18 @@ Bool __move_loop(
 			/* check Paging request once and only once after
 			 * outline redrawn
 			 * redraw after paging if needed - mab */
-			paged = 0;
-			while (paged <= 1)
+			for (paged = 0; paged <= 1; paged++)
 			{
 				if (!do_move_opaque)
+				{
 					draw_move_resize_grid(
 						xl, yt, Width - 1, Height - 1);
+				}
 				else
 				{
 					if (IS_ICONIFIED(fw))
 					{
-						set_icon_position(
-							fw, xl, yt);
+						set_icon_position(fw, xl, yt);
 						move_icon_to_position(fw);
 						broadcast_icon_geometry(
 							fw, False);
@@ -2270,8 +2356,8 @@ Bool __move_loop(
 					else
 					{
 						XMoveWindow(
-							dpy, FW_W_FRAME(fw), xl,
-							yt);
+							dpy, FW_W_FRAME(fw),
+							xl, yt);
 					}
 				}
 				DisplayPosition(fw, &e, xl, yt, False);
@@ -2280,27 +2366,25 @@ Bool __move_loop(
 				 * when paging - mab */
 				if (paged == 0)
 				{
+					XEvent le;
+
 					xl = e.xmotion.x_root;
 					yt = e.xmotion.y_root;
+					fev_get_last_event(&le);
 					HandlePaging(
-						&e, dx, dy, &xl, &yt, &delta_x,
-						&delta_y, False, False, False);
+						&le, dx, dy, &xl, &yt,
+						&delta_x, &delta_y, False,
+						False, False);
 					if (delta_x)
 					{
 						x_virtual_offset = 0;
 					}
-					if (XOffset)
-					{
-						xl += XOffset;
-					}
+					xl += XOffset;
 					if (delta_y)
 					{
 						y_virtual_offset = 0;
 					}
-					if (YOffset)
-					{
-						yt += YOffset;
-					}
+					yt += YOffset;
 					if (do_snap)
 					{
 						DoSnapAttract(
@@ -2314,8 +2398,7 @@ Bool __move_loop(
 						break;
 					}
 				}
-				paged++;
-			}  /* end while (paged) */
+			}
 			break;
 
 		case Expose:
@@ -2723,15 +2806,135 @@ void CMD_XorPixmap(F_CMD_ARGS)
 
 /* ----------------------------- resizing code ----------------------------- */
 
-/*
- *
- *  Procedure:
+static void __resize_get_dir_from_window(
+	int *ret_xmotion, int *ret_ymotion, FvwmWindow *fw, Window context_w)
+{
+	if (context_w != Scr.Root && context_w != None)
+	{
+		if (context_w == FW_W_SIDE(fw, 0))   /* top */
+		{
+			*ret_ymotion = 1;
+		}
+		else if (context_w == FW_W_SIDE(fw, 1))  /* right */
+		{
+			*ret_xmotion = -1;
+		}
+		else if (context_w == FW_W_SIDE(fw, 2))  /* bottom */
+		{
+			*ret_ymotion = -1;
+		}
+		else if (context_w == FW_W_SIDE(fw, 3))  /* left */
+		{
+			*ret_xmotion = 1;
+		}
+		else if (context_w == FW_W_CORNER(fw, 0))  /* upper-left */
+		{
+			*ret_xmotion = 1;
+			*ret_ymotion = 1;
+		}
+		else if (context_w == FW_W_CORNER(fw, 1))  /* upper-right */
+		{
+			*ret_xmotion = -1;
+			*ret_ymotion = 1;
+		}
+		else if (context_w == FW_W_CORNER(fw, 2)) /* lower left */
+		{
+			*ret_xmotion = 1;
+			*ret_ymotion = -1;
+		}
+		else if (context_w == FW_W_CORNER(fw, 3))  /* lower right */
+		{
+			*ret_xmotion = -1;
+			*ret_ymotion = -1;
+		}
+	}
+
+	return;
+}
+
+static void __resize_get_dir_proximity(
+	int *ret_xmotion, int *ret_ymotion, FvwmWindow *fw, int x_off,
+	int y_off, int px, int py)
+{
+	int tx;
+	int ty;
+
+	if (px < 0 || x_off < 0 || py < 0 || y_off < 0)
+	{
+		return;
+	}
+	/* Now find the place to warp to. We simply use the sectors
+	 * drawn when we start resizing the window. */
+#if 0
+	tx = orig->width / 10 - 1;
+	ty = orig->height / 10 - 1;
+#else
+	tx = 0;
+	ty = 0;
+#endif
+	tx = max(fw->boundary_width, tx);
+	ty = max(fw->boundary_width, ty);
+	if (px < tx)
+	{
+		*ret_xmotion = 1;
+	}
+	else if (x_off < tx)
+	{
+		*ret_xmotion = -1;
+	}
+	if (py < ty)
+	{
+		*ret_ymotion = 1;
+	}
+	else if (y_off < ty)
+	{
+		*ret_ymotion = -1;
+	}
+
+	return;
+}
+
+static void __resize_get_refpos(
+	int *ret_x, int *ret_y, int xmotion, int ymotion, int w, int h,
+	FvwmWindow *fw)
+{
+	if (xmotion > 0)
+	{
+		*ret_x = 0;
+	}
+	else if (xmotion < 0)
+	{
+		*ret_x = w - 1;
+	}
+	else
+	{
+		*ret_x = w / 2;
+	}
+	if (ymotion > 0)
+	{
+		*ret_y = 0;
+	}
+	else if (ymotion < 0)
+	{
+		*ret_y = h - 1;
+	}
+	else
+	{
+		*ret_y = h / 2;
+	}
+
+	return;
+}
+
+/* Procedure:
  *      __resize_step - move the rubberband around.  This is called for
  *                 each motion event when we are resizing
  *
  *  Inputs:
  *      x_root   - the X corrdinate in the root window
  *      y_root   - the Y corrdinate in the root window
+ *      x_off    - x offset of pointer from border (input/output)
+ *      y_off    - y offset of pointer from border (input/output)
  *      fw       - the current fvwm window
  *      drag     - resize internal structure
  *      orig     - resize internal structure
@@ -2740,44 +2943,110 @@ void CMD_XorPixmap(F_CMD_ARGS)
  *
  */
 static void __resize_step(
-	const exec_context_t *exc, int x_root, int y_root, rectangle *drag,
-	rectangle *orig, int *xmotionp, int *ymotionp, Bool do_resize_opaque)
+	const exec_context_t *exc, int x_root, int y_root, int *x_off,
+	int *y_off, rectangle *drag, rectangle *orig, int *xmotionp,
+	int *ymotionp, Bool do_resize_opaque, Bool is_direction_fixed)
 {
 	int action = 0;
 	XEvent e;
+	int x2;
+	int y2;
+	int xdir;
+	int ydir;
 
-	if ((y_root <= orig->y) ||
-	    ((*ymotionp == 1)&&(y_root < orig->y+orig->height-1)))
+	x2 = x_root - *x_off;
+	x_root += *x_off;
+	if (is_direction_fixed == True && (*xmotionp != 0 || *ymotionp != 0))
 	{
-		drag->y = y_root;
-		drag->height = orig->y + orig->height - y_root;
-		action = 1;
-		*ymotionp = 1;
+		xdir = *xmotionp;
 	}
-	else if ((y_root >= orig->y + orig->height - 1)||
-		 ((*ymotionp == -1)&&(y_root > orig->y)))
+	else if (x2 <= orig->x ||
+		 (*xmotionp == 1 && x2 < orig->x + orig->width - 1))
 	{
-		drag->y = orig->y;
-		drag->height = 1 + y_root - drag->y;
-		action = 1;
-		*ymotionp = -1;
+		xdir = 1;
 	}
-
-	if ((x_root <= orig->x)||
-	    ((*xmotionp == 1)&&(x_root < orig->x + orig->width - 1)))
+	else if (x2 >= orig->x + orig->width - 1 ||
+		 (*xmotionp == -1 && x2 > orig->x))
 	{
+		xdir = -1;
+	}
+	else
+	{
+		xdir = 0;
+	}
+	switch (xdir)
+	{
+	case 1:
+		if (*xmotionp != 1)
+		{
+			*x_off = -*x_off;
+			x_root = x2;
+			*xmotionp = 1;
+		}
 		drag->x = x_root;
 		drag->width = orig->x + orig->width - x_root;
 		action = 1;
-		*xmotionp = 1;
-	}
-	if ((x_root >= orig->x + orig->width - 1)||
-	    ((*xmotionp == -1)&&(x_root > orig->x)))
-	{
+		break;
+	case -1:
+		if (*xmotionp != -1)
+		{
+			*x_off = -*x_off;
+			x_root = x2;
+			*xmotionp = -1;
+		}
 		drag->x = orig->x;
-		drag->width = 1 + x_root - orig->x;
+		drag->width = 1 + x_root - drag->x;
 		action = 1;
-		*xmotionp = -1;
+		break;
+	default:
+		break;
+	}
+	y2 = y_root - *y_off;
+	y_root += *y_off;
+	if (is_direction_fixed == True && (*xmotionp != 0 || *ymotionp != 0))
+	{
+		ydir = *ymotionp;
+	}
+	else if (y2 <= orig->y ||
+		 (*ymotionp == 1 && y2 < orig->y + orig->height - 1))
+	{
+		ydir = 1;
+	}
+	else if (y2 >= orig->y + orig->height - 1 ||
+		 (*ymotionp == -1 && y2 > orig->y))
+	{
+		ydir = -1;
+	}
+	else
+	{
+		ydir = 0;
+	}
+	switch (ydir)
+	{
+	case 1:
+		if (*ymotionp != 1)
+		{
+			*y_off = -*y_off;
+			y_root = y2;
+			*ymotionp = 1;
+		}
+		drag->y = y_root;
+		drag->height = orig->y + orig->height - y_root;
+		action = 1;
+		break;
+	case -1:
+		if (*ymotionp != -1)
+		{
+			*y_off = -*y_off;
+			y_root = y2;
+			*ymotionp = -1;
+		}
+		drag->y = orig->y;
+		drag->height = 1 + y_root - drag->y;
+		action = 1;
+		break;
+	default:
+		break;
 	}
 
 	if (action)
@@ -2812,7 +3081,7 @@ static void __resize_step(
 	e.type = MotionNotify;
 	e.xbutton.x_root = x_root;
 	e.xbutton.y_root = y_root;
-	DisplaySize(exc->w.fw, &e, drag->width, drag->height,False,False);
+	DisplaySize(exc->w.fw, &e, drag->width, drag->height, False, False);
 
 	return;
 }
@@ -2822,11 +3091,15 @@ static Bool __resize_window(F_CMD_ARGS)
 {
 	extern Window bad_window;
 	Bool finished = False, is_done = False, is_aborted = False;
+	Bool do_send_cn = False;
 	Bool do_resize_opaque;
-	int x,y,delta_x,delta_y,stashed_x,stashed_y;
-	Window ResizeWindow;
+	Bool do_warp_to_border;
+	Bool is_direction_fixed;
 	Bool fButtonAbort = False;
 	Bool fForceRedraw = False;
+	Bool called_from_title = False;
+	int x,y,delta_x,delta_y,stashed_x,stashed_y;
+	Window ResizeWindow;
 	int n;
 	unsigned int button_mask = 0;
 	rectangle sdrag;
@@ -2843,19 +3116,27 @@ static Bool __resize_window(F_CMD_ARGS)
 	int py;
 	int i;
 	size_borders b;
-	Bool called_from_title = False;
 	frame_move_resize_args mr_args = NULL;
 	long evmask;
 	FvwmWindow *fw = exc->w.fw;
 	XEvent ev;
+	int ref_x;
+	int ref_y;
+	int x_off;
+	int y_off;
+	direction_t dir;
 
 	bad_window = False;
 	ResizeWindow = FW_W_FRAME(fw);
-	if (FQueryPointer(
-		    dpy, ResizeWindow, &JunkRoot, &JunkChild, &JunkX, &JunkY,
-		    &px, &py, &button_mask) == False)
+	if (fev_get_evpos_or_query(dpy, Scr.Root, exc->x.etrigger, &px, &py) ==
+	    False ||
+	    XTranslateCoordinates(
+		    dpy, Scr.Root, ResizeWindow, px, py, &px, &py,
+		    &JunkChild) == False)
 	{
 		/* pointer is on a different screen - that's okay here */
+		px = 0;
+		py = 0;
 	}
 	button_mask &= DEFAULT_ALL_BUTTONS_MASK;
 
@@ -2895,7 +3176,8 @@ static Bool __resize_window(F_CMD_ARGS)
 		&action, fw->frame_g.x, fw->frame_g.y,
 		fw->hints.base_width, fw->hints.base_height,
 		fw->hints.width_inc, fw->hints.height_inc,
-		&b, &(drag->width), &(drag->height));
+		&b, &(drag->width), &(drag->height),
+		&dir, &is_direction_fixed, &do_warp_to_border);
 
 	if (n == 2)
 	{
@@ -2995,58 +3277,27 @@ static Bool __resize_window(F_CMD_ARGS)
 	}
 	DisplaySize(fw, exc->x.elast, orig->width, orig->height,True,True);
 
-	if ((PressedW != Scr.Root)&&(PressedW != None))
+	if (dir != DIR_NONE)
 	{
-		/* Get the current position to determine which border to resize
-		 */
-		if (PressedW == FW_W_SIDE(fw, 0))   /* top */
-		{
-			ymotion = 1;
-		}
-		else if (PressedW == FW_W_SIDE(fw, 1))  /* right */
-		{
-			xmotion = -1;
-		}
-		else if (PressedW == FW_W_SIDE(fw, 2))  /* bottom */
-		{
-			ymotion = -1;
-		}
-		else if (PressedW == FW_W_SIDE(fw, 3))  /* left */
-		{
-			xmotion = 1;
-		}
-		else if (PressedW == FW_W_CORNER(fw, 0))  /* upper-left */
-		{
-			ymotion = 1;
-			xmotion = 1;
-		}
-		else if (PressedW == FW_W_CORNER(fw, 1))  /* upper-right */
-		{
-			xmotion = -1;
-			ymotion = 1;
-		}
-		else if (PressedW == FW_W_CORNER(fw, 2)) /* lower left */
-		{
-			ymotion = -1;
-			xmotion = 1;
-		}
-		else if (PressedW == FW_W_CORNER(fw, 3))  /* lower right */
-		{
-			ymotion = -1;
-			xmotion = -1;
-		}
-	}
+		int grav;
 
-	/* begin of code responsible for warping the pointer to the border when
-	 * starting a resize. */
+		grav = gravity_dir_to_grav(dir);
+		gravity_get_offsets(grav, &xmotion, &ymotion);
+		xmotion = -xmotion;
+		ymotion = -ymotion;
+	}
+	if (xmotion == 0 && ymotion == 0)
+	{
+		__resize_get_dir_from_window(&xmotion, &ymotion, fw, PressedW);
+	}
 	if (FW_W_TITLE(fw) != None && PressedW == FW_W_TITLE(fw))
 	{
-		/* title was pressed to thart the resize */
+		/* title was pressed to start the resize */
 		called_from_title = True;
 	}
 	else
 	{
-		for (i = NUMBER_OF_BUTTONS; i--; )
+		for (i = NUMBER_OF_TITLE_BUTTONS; i--; )
 		{
 			/* see if the title button was pressed to that the
 			 * resize */
@@ -3063,103 +3314,42 @@ static Bool __resize_window(F_CMD_ARGS)
 	if (PressedW != Scr.Root && xmotion == 0 && ymotion == 0 &&
 	    !called_from_title)
 	{
-		int dx = orig->width - px;
-		int dy = orig->height - py;
-		int wx = -1;
-		int wy = -1;
-		int tx;
-		int ty;
-
-		/* Now find the place to warp to. We simply use the sectors
-		 * drawn when we start resizing the window. */
-#if 0
-		tx = orig->width / 10 - 1;
-		ty = orig->height / 10 - 1;
-#else
-		tx = 0;
-		ty = 0;
-#endif
-		tx = max(fw->boundary_width, tx);
-		ty = max(fw->boundary_width, ty);
-		if (px >= 0 && dx >= 0 && py >= 0 && dy >= 0)
+		__resize_get_dir_proximity(
+			&xmotion, &ymotion, fw, orig->width - px,
+			orig->height - py, px, py);
+		if (xmotion != 0 || ymotion != 0)
 		{
-			if (px < tx)
-			{
-				if (py < ty)
-				{
-					xmotion = 1;
-					ymotion = 1;
-					wx = 0;
-					wy = 0;
-				}
-				else if (dy < ty)
-				{
-					xmotion = 1;
-					ymotion = -1;
-					wx = 0;
-					wy = orig->height -1;
-				}
-				else
-				{
-					xmotion = 1;
-					wx = 0;
-					wy = orig->height/2;
-					wy = py;
-				}
-			}
-			else if (dx < tx)
-			{
-				if (py < ty)
-				{
-					xmotion = -1;
-					ymotion = 1;
-					wx = orig->width - 1;
-					wy = 0;
-				}
-				else if (dy < ty)
-				{
-					xmotion = -1;
-					ymotion = -1;
-					wx = orig->width - 1;
-					wy = orig->height -1;
-				}
-				else
-				{
-					xmotion = -1;
-					wx = orig->width - 1;
-					wy = orig->height/2;
-					wy = py;
-				}
-			}
-			else
-			{
-				if (py < ty)
-				{
-					ymotion = 1;
-					wx = orig->width/2;
-					wy = 0;
-					wx = px;
-				}
-				else if (dy < ty)
-				{
-					ymotion = -1;
-					wx = orig->width/2;
-					wy = orig->height -1;
-					wx = px;
-				}
-			}
-		}
-
-		if (wx != -1)
-		{
-			/* now warp the pointer to the border */
-			FWarpPointer(
-				dpy, None, ResizeWindow, 0, 0, 1, 1, wx, wy);
-			XFlush(dpy);
+			do_warp_to_border = True;
 		}
 	}
-	/* end of code responsible for warping the pointer to the border when
-	 * starting a resize. */
+	__resize_get_refpos(
+		&ref_x, &ref_y, xmotion, ymotion, orig->width, orig->height,
+		fw);
+	x_off = 0;
+	y_off = 0;
+	if (do_warp_to_border == True)
+	{
+		int dx;
+		int dy;
+
+		dx = (xmotion == 0) ? px : ref_x;
+		dy = (ymotion == 0) ? py : ref_y;
+		/* warp the pointer to the border */
+		FWarpPointer(
+			dpy, None, ResizeWindow, 0, 0, 1, 1, dx, dy);
+		XFlush(dpy);
+	}
+	else if (xmotion != 0 || ymotion != 0)
+	{
+		/* keep the distance between pointer and border */
+		x_off = (xmotion == 0) ? 0 : ref_x - px;
+		y_off = (ymotion == 0) ? 0 : ref_y - py;
+	}
+	else
+	{
+		/* wait until the pointer hits a border before making a
+		 * decision about the resize direction */
+	}
 
 	/* draw the rubber-band window */
 	if (!do_resize_opaque)
@@ -3171,6 +3361,9 @@ static Bool __resize_window(F_CMD_ARGS)
 	 * press */
 	if (exc->x.elast->type == KeyPress)
 	{
+		int xo;
+		int yo;
+
 		if (FQueryPointer(
 			    dpy, Scr.Root, &JunkRoot, &JunkChild, &stashed_x,
 			    &stashed_y, &JunkX, &JunkY, &JunkMask) == False)
@@ -3179,9 +3372,11 @@ static Bool __resize_window(F_CMD_ARGS)
 			stashed_x = 0;
 			stashed_y = 0;
 		}
+		xo = 0;
+		yo = 0;
 		__resize_step(
-			exc, stashed_x, stashed_y, drag, orig, &xmotion,
-			&ymotion, do_resize_opaque);
+			exc, stashed_x, stashed_y, &xo, &yo, drag, orig,
+			&xmotion, &ymotion, do_resize_opaque, True);
 	}
 	else
 	{
@@ -3212,23 +3407,95 @@ static Bool __resize_window(F_CMD_ARGS)
 		}
 		if (rc == -1)
 		{
-			FMaskEvent(dpy, evmask, &ev);
+			FMaskEvent(
+				dpy,
+				evmask | EnterWindowMask | LeaveWindowMask,
+				&ev);
 		}
-		if (ev.type == MotionNotify)
+		if (ev.type == MotionNotify ||
+		    ev.type == EnterNotify || ev.type == LeaveNotify)
 		{
+			Bool is_motion;
+
+			is_motion = (ev.type == MotionNotify) ? True : False;
 			/* discard any extra motion events before a release */
-			while (FCheckMaskEvent(
-				       dpy, ButtonMotionMask |
-				       PointerMotionMask | ButtonReleaseMask |
-				       ButtonPressMask, &ev))
+			/* dv (2004-07-01): With XFree 4.1.0.1, some Mouse
+			 * events are not reported to fvwm when the pointer
+			 * moves very fast and suddenly stops in the corner of
+			 * the screen.  Handle EnterNotify/LeaveNotify events
+			 * too to get an idea where the pointer might be. */
+			while (
+				FCheckMaskEvent(
+					dpy, ButtonMotionMask |
+					PointerMotionMask | ButtonReleaseMask |
+					ButtonPressMask | EnterWindowMask |
+					LeaveWindowMask, &ev) == True)
 			{
 				if (ev.type == ButtonRelease ||
-				    ev.type == ButtonPress)
+				    ev.type == ButtonPress ||
+				    ev.type == KeyPress)
 				{
 					break;
 				}
 			}
 		}
+		if (ev.type == EnterNotify || ev.type == LeaveNotify)
+		{
+			XEvent e2;
+			int x;
+			int y;
+
+			/* Query the pointer to catch the latest information.
+			 * This *is* necessary. */
+			if (FQueryPointer(
+				    dpy, Scr.Root, &JunkRoot, &JunkChild, &x,
+				    &y, &JunkX, &JunkY, &JunkMask) == True)
+			{
+				fev_make_null_event(&e2, dpy);
+				e2.type = MotionNotify;
+				e2.xmotion.time = fev_get_evtime();
+				e2.xmotion.x_root = x;
+				e2.xmotion.y_root = y;
+				e2.xmotion.state = JunkMask;
+				e2.xmotion.same_screen = True;
+				ev = e2;
+				fev_fake_event(&ev);
+			}
+			else
+			{
+				/* pointer is on a different screen,
+				 * ignore event */
+			}
+		}
+		if (ev.type == EnterNotify || ev.type == LeaveNotify)
+		{
+			XEvent e2;
+			int x;
+			int y;
+
+			/* Query the pointer to catch the latest information.
+			 * This *is* necessary. */
+			if (FQueryPointer(
+				    dpy, Scr.Root, &JunkRoot, &JunkChild, &x,
+				    &y, &JunkX, &JunkY, &button_mask) ==
+			    True)
+			{
+				fev_make_null_event(&e2, dpy);
+				e2.type = MotionNotify;
+				e2.xmotion.time = fev_get_evtime();
+				e2.xmotion.x_root = x;
+				e2.xmotion.y_root = y;
+				e2.xmotion.state = JunkMask;
+				e2.xmotion.same_screen = True;
+				ev = e2;
+				fev_fake_event(&ev);
+			}
+			else
+			{
+				/* pointer is on a different screen,
+				 * ignore event */
+			}
+		}
 
 		is_done = False;
 		/* Handle a limited number of key press events to allow
@@ -3236,6 +3503,10 @@ static Bool __resize_window(F_CMD_ARGS)
 		if (ev.type == KeyPress)
 		{
 			Keyboard_shortcuts(&ev, fw, NULL, NULL, ButtonRelease);
+			if (ev.type == ButtonRelease)
+			{
+				do_send_cn = True;
+			}
 		}
 		switch (ev.type)
 		{
@@ -3257,13 +3528,21 @@ static Bool __resize_window(F_CMD_ARGS)
 			if (button_mask || (ev.xbutton.button != 1))
 			{
 				fButtonAbort = True;
+				/* fall through */
+			}
+			else
+			{
+				finished = True;
+				do_send_cn = True;
+				break;
 			}
 		case KeyPress:
 			/* simple code to bag out of move - CKH */
-			if (XLookupKeysym(&ev.xkey, 0) == XK_Escape ||
-			    fButtonAbort)
+			if (fButtonAbort ||
+			    XLookupKeysym(&ev.xkey, 0) == XK_Escape)
 			{
 				is_aborted = True;
+				do_send_cn = True;
 				finished = True;
 				/* return pointer if aborted resize was invoked
 				 * with key */
@@ -3281,10 +3560,18 @@ static Bool __resize_window(F_CMD_ARGS)
 				}
 				if (do_resize_opaque)
 				{
+					int xo;
+					int yo;
+
+					xo = 0;
+					yo = 0;
+					xmotion = 1;
+					ymotion = 1;
 					__resize_step(
 						exc, start_g.x, start_g.y,
-						&start_g, orig, &xmotion,
-						&ymotion, do_resize_opaque);
+						&xo, &yo, &start_g, orig,
+						&xmotion, &ymotion,
+						do_resize_opaque, True);
 				}
 			}
 			is_done = True;
@@ -3307,8 +3594,9 @@ static Bool __resize_window(F_CMD_ARGS)
 				/* resize before paging request to prevent
 				 * resize from lagging * mouse - mab */
 				__resize_step(
-					exc, x, y, drag, orig, &xmotion,
-					&ymotion, do_resize_opaque);
+					exc, x, y, &x_off, &y_off, drag, orig,
+					&xmotion, &ymotion, do_resize_opaque,
+					is_direction_fixed);
 				/* need to move the viewport */
 				HandlePaging(
 					&ev, Scr.EdgeScrollX, Scr.EdgeScrollY,
@@ -3324,8 +3612,9 @@ static Bool __resize_window(F_CMD_ARGS)
 				drag->y -= delta_y;
 
 				__resize_step(
-					exc, x, y, drag, orig, &xmotion,
-					&ymotion, do_resize_opaque);
+					exc, x, y, &x_off, &y_off, drag, orig,
+					&xmotion, &ymotion, do_resize_opaque,
+					is_direction_fixed);
 			}
 			fForceRedraw = False;
 			is_done = True;
@@ -3451,9 +3740,21 @@ static Bool __resize_window(F_CMD_ARGS)
 	{
 		frame_free_move_resize_args(fw, mr_args);
 	}
+	if (do_send_cn == True)
+	{
+		rectangle g;
+
+		if (is_aborted)
+		{
+			g = start_g;
+		}
+		else
+		{
+			g = *drag;
+		}
+		SendConfigureNotify(fw, g.x, g.y, g.width, g.height, 0, True);
+	}
 	MyXUngrabKeyboard(dpy);
-	xmotion = 0;
-	ymotion = 0;
 	WaitForButtonsUp(True);
 	UngrabEm(GRAB_NORMAL);
 	Scr.flags.do_edge_wrap_x = edge_wrap_x;
@@ -3561,7 +3862,7 @@ static void move_sticky_window_to_same_page(
 static void MaximizeHeight(
 	FvwmWindow *win, unsigned int win_width, int win_x,
 	unsigned int *win_height, int *win_y, Bool grow_up, Bool grow_down,
-	int top_border, int bottom_border, Bool layer_grow)
+	int top_border, int bottom_border, int *layers)
 {
 	FvwmWindow *cwin;
 	int x11, x12, x21, x22;
@@ -3581,8 +3882,12 @@ static void MaximizeHeight(
 	{
 		if (cwin == win ||
 		    (cwin->Desk != win->Desk &&
-		     !is_window_sticky_across_desks(cwin)) ||
-		    (layer_grow && cwin->layer <= win->layer))
+		     !is_window_sticky_across_desks(cwin)))
+		{
+			continue;
+		}
+		if ((layers[0] >= 0 && cwin->layer < layers[0]) ||
+		    (layers[1] >= 0 && cwin->layer > layers[1]))
 		{
 			continue;
 		}
@@ -3632,7 +3937,7 @@ static void MaximizeHeight(
 static void MaximizeWidth(
 	FvwmWindow *win, unsigned int *win_width, int *win_x,
 	unsigned int win_height, int win_y, Bool grow_left, Bool grow_right,
-	int left_border, int right_border, Bool layer_grow)
+	int left_border, int right_border, int *layers)
 {
 	FvwmWindow *cwin;
 	int x11, x12, x21, x22;
@@ -3652,8 +3957,12 @@ static void MaximizeWidth(
 	{
 		if (cwin == win ||
 		    (cwin->Desk != win->Desk &&
-		     !is_window_sticky_across_desks(cwin)) ||
-		    (layer_grow && cwin->layer <= win->layer))
+		     !is_window_sticky_across_desks(cwin)))
+		{
+			continue;
+		}
+		if ((layers[0] >= 0 && cwin->layer < layers[0]) ||
+		    (layers[1] >= 0 && cwin->layer > layers[1]))
 		{
 			continue;
 		}
@@ -3713,6 +4022,8 @@ static void unmaximize_fvwm_window(
 	}
 	frame_setup_window(
 		fw, new_g.x, new_g.y, new_g.width, new_g.height, True);
+	border_draw_decorations(
+		fw, PART_ALL, (Scr.Hilite == fw), True, CLEAR_ALL, NULL, NULL);
 	if (IS_EWMH_FULLSCREEN(fw))
 	{
 		SET_EWMH_FULLSCREEN(fw, False);
@@ -3777,10 +4088,9 @@ void CMD_Maximize(F_CMD_ARGS)
 	Bool do_force_maximize = False;
 	Bool is_screen_given = False;
 	Bool ignore_working_area = False;
-	Bool layer_grow = False;
+	int layers[2] = { -1, -1 };
 	Bool global_flag_parsed = False;
 	int  scr_x, scr_y, scr_w, scr_h;
-	int sx, sy, sw, sh;
 	rectangle new_g;
 	FvwmWindow *fw = exc->w.fw;
 
@@ -3816,11 +4126,24 @@ void CMD_Maximize(F_CMD_ARGS)
 				ignore_working_area = True;
 				action = taction;
 			}
-			else if (StrEquals(token, "layer"))
+			else if (StrEquals(token, "growonwindowlayer"))
 			{
-				layer_grow = True;
+				layers[0] = fw->layer;
+				layers[1] = fw->layer;
 				action = taction;
 			}
+			else if (StrEquals(token, "growonlayers"))
+			{
+				int n;
+
+				n = GetIntegerArguments(
+					taction, &action, layers, 2);
+				if (n != 2)
+				{
+					layers[0] = -1;
+					layers[1] = -1;
+				}
+			}
 			else
 			{
 				global_flag_parsed = True;
@@ -3857,14 +4180,11 @@ void CMD_Maximize(F_CMD_ARGS)
 			&fscr, FSCREEN_XYPOS, &scr_x, &scr_y, &scr_w, &scr_h);
 	}
 
-	sx = scr_x;
-	sy = scr_y;
-	sw = scr_w;
-	sh = scr_h;
 	if (!ignore_working_area)
 	{
 		EWMH_GetWorkAreaIntersection(
-			fw, &sx, &sy, &sw, &sh, EWMH_MAXIMIZE_MODE(fw));
+			fw, &scr_x, &scr_y, &scr_w, &scr_h,
+			EWMH_MAXIMIZE_MODE(fw));
 	}
 #if 0
 	fprintf(stderr, "%s: page=(%d,%d), scr=(%d,%d, %dx%d)\n", __FUNCTION__,
@@ -3872,7 +4192,7 @@ void CMD_Maximize(F_CMD_ARGS)
 #endif
 
 	/* parse first parameter */
-	val1_unit = sw;
+	val1_unit = scr_w;
 	token = PeekToken(action, &taction);
 	if (token && StrEquals(token, "grow"))
 	{
@@ -3898,24 +4218,24 @@ void CMD_Maximize(F_CMD_ARGS)
 		if (GetOnePercentArgument(token, &val1, &val1_unit) == 0)
 		{
 			val1 = 100;
-			val1_unit = sw;
+			val1_unit = scr_w;
 		}
 		else if (val1 < 0)
 		{
 			/* handle negative offsets */
-			if (val1_unit == sw)
+			if (val1_unit == scr_w)
 			{
 				val1 = 100 + val1;
 			}
 			else
 			{
-				val1 = sw + val1;
+				val1 = scr_w + val1;
 			}
 		}
 	}
 
 	/* parse second parameter */
-	val2_unit = sh;
+	val2_unit = scr_h;
 	token = PeekToken(taction, NULL);
 	if (token && StrEquals(token, "grow"))
 	{
@@ -3941,33 +4261,22 @@ void CMD_Maximize(F_CMD_ARGS)
 		if (GetOnePercentArgument(token, &val2, &val2_unit) == 0)
 		{
 			val2 = 100;
-			val2_unit = sh;
+			val2_unit = scr_h;
 		}
 		else if (val2 < 0)
 		{
 			/* handle negative offsets */
-			if (val2_unit == sh)
+			if (val2_unit == scr_h)
 			{
 				val2 = 100 + val2;
 			}
 			else
 			{
-				val2 = sh + val2;
+				val2 = scr_h + val2;
 			}
 		}
 	}
 
-	if (!grow_left && !grow_right)
-	{
-		scr_x = sx;
-		scr_w = sw;
-	}
-	if (!grow_up && !grow_down)
-	{
-		scr_y = sy;
-		scr_h = sh;
-	}
-
 #if 0
 	fprintf(stderr, "%s: page=(%d,%d), scr=(%d,%d, %dx%d)\n", __FUNCTION__,
 		page_x, page_y, scr_x, scr_y, scr_w, scr_h);
@@ -3986,7 +4295,7 @@ void CMD_Maximize(F_CMD_ARGS)
 				fw, new_g.width, new_g.x,
 				(unsigned int *)&new_g.height, &new_g.y,
 				grow_up, grow_down, page_y + scr_y,
-				page_y + scr_y + scr_h, layer_grow);
+				page_y + scr_y + scr_h, layers);
 		}
 		else if (val2 > 0)
 		{
@@ -3999,7 +4308,7 @@ void CMD_Maximize(F_CMD_ARGS)
 				fw, (unsigned int *)&new_g.width, &new_g.x,
 				new_g.height, new_g.y, grow_left, grow_right,
 				page_x + scr_x, page_x + scr_x + scr_w,
-				layer_grow);
+				layers);
 		}
 		else if (val1 >0)
 		{
