@@ -82,6 +82,7 @@ typedef struct _match
 	int max_x_offset, max_y_offset;
 	int desktop;
 	int layer;
+	int default_layer;
 	int used;
 	int gravity;
 	unsigned long ewmh_hint_desktop;
@@ -293,7 +294,7 @@ GetClientID(FvwmWindow *fw)
 		if (actual_type == XA_WINDOW && actual_format == 32 &&
 		    nitems == 1 && bytes_after == 0)
 		{
-			client_leader = *((Window *) prop);
+			client_leader = (Window)(*(long *)prop);
 		}
 	}
 
@@ -305,7 +306,9 @@ GetClientID(FvwmWindow *fw)
 
 	if (client_leader)
 	{
-		if (XGetTextProperty(dpy, client_leader, &tp, _XA_SM_CLIENT_ID))
+		if (
+			XGetTextProperty(
+				dpy, client_leader, &tp, _XA_SM_CLIENT_ID))
 		{
 			if (tp.encoding == XA_STRING && tp.format == 8 &&
 			    tp.nitems != 0)
@@ -452,6 +455,10 @@ SaveWindowStates(FILE *f)
 			fprintf(f, "  [WINDOW_ROLE] %s\n", window_role);
 			XFree(window_role);
 		}
+		if (client_id && window_role)
+		{
+			/* we have enough information */
+		}
 		else
 		{
 			if (ewin->class.res_class)
@@ -517,7 +524,7 @@ SaveWindowStates(FILE *f)
 		{
 			layer = Scr.DefaultLayer;
 		}
-		fprintf(f, "  [LAYER] %i\n", layer);
+		fprintf(f, "  [LAYER] %i %i\n", layer, ewin->default_layer);
 		fprintf(f, "  [EWMH_DESKTOP] %lu\n", ewin->ewmh_hint_desktop);
 		fprintf(f, "  [FLAGS] ");
 		for (i = 0; i < sizeof(window_flags); i++)
@@ -542,7 +549,14 @@ static Bool matchWin(FvwmWindow *w, Match *m)
 	found = 0;
 	client_id = GetClientID(w);
 
-	if (xstreq(client_id, m->client_id))
+	if (Restarting)
+	{
+		if (FW_W(w) == m->win)
+		{
+			found = 1;
+		}
+	}
+	else if (xstreq(client_id, m->client_id))
 	{
 
 		/* client_id's match */
@@ -570,7 +584,6 @@ static Bool matchWin(FvwmWindow *w, Match *m)
 			{
 				/* for non-SM-aware clients we also
 				 * compare WM_COMMAND */
-
 				if (!XGetCommand(
 					    dpy, FW_W(w), &wm_command,
 					    &wm_command_count))
@@ -595,11 +608,10 @@ static Bool matchWin(FvwmWindow *w, Match *m)
 						/* migo (21/Oct/1999):
 						 * on restarts compare
 						 * window ids too */
-						if (!Restarting ||
-						    FW_W(w) == m->win)
-						{
-							found = 1;
-						}
+						/* But if we restart we only need
+						 * to compare window ids
+						 * olicha (2005-01-06) */
+						found = 1;
 					}
 				} /* if (wm_command_count ==... */
 			} /* else if res_class, res_name and wm_name agree */
@@ -1283,6 +1295,7 @@ LoadWindowStates(char *filename)
 			matches[num_match - 1].icon_y = 0;
 			matches[num_match - 1].desktop = 0;
 			matches[num_match - 1].layer = 0;
+			matches[num_match - 1].default_layer = 0;
 			memset(&(matches[num_match - 1].flags), 0,
 			       sizeof(window_flags));
 			matches[num_match - 1].used = 0;
@@ -1314,8 +1327,9 @@ LoadWindowStates(char *filename)
 		}
 		else if (!strcmp(s1, "[LAYER]"))
 		{
-			sscanf(s, "%*s %i",
-			       &(matches[num_match - 1].layer));
+			sscanf(s, "%*s %i %i",
+			       &(matches[num_match - 1].layer),
+			       &(matches[num_match - 1].default_layer));
 		}
 		else if (!strcmp(s1, "[EWMH_DESKTOP]"))
 		{
@@ -1533,6 +1547,7 @@ MatchWinToSM(
 			ewin->Desk = (IS_STICKY_ACROSS_DESKS(ewin)) ?
 				Scr.CurrentDesk : matches[i].desktop;
 			set_layer(ewin, matches[i].layer);
+			set_default_layer(ewin, matches[i].default_layer);
 			/* Note: the Modal, skip pager, skip taskbar and
 			 * "stacking order" state are not restored here: there
 			 * are restored in EWMH_ExitStuff */
