@@ -111,6 +111,7 @@ static int GetDeskNumber(char *action, int current_desk)
 {
 	int n;
 	int m;
+	int is_relative;
 	int desk;
 	int val[4];
 	int min, max;
@@ -119,31 +120,29 @@ static int GetDeskNumber(char *action, int current_desk)
 	{
 		return prev_desk;
 	}
-
 	n = GetIntegerArguments(action, NULL, &(val[0]), 4);
 	if (n <= 0)
 	{
-		return current_desk;
+		return Scr.CurrentDesk;
 	}
 	if (n == 1)
 	{
 		return current_desk + val[0];
 	}
-
 	desk = current_desk;
 	m = 0;
-
 	if (val[0] == 0)
 	{
 		/* absolute desk number */
 		desk = val[1];
+		is_relative = 0;
 	}
 	else
 	{
 		/* relative desk number */
 		desk += val[0];
+		is_relative = 1;
 	}
-
 	if (n == 3)
 	{
 		m = 1;
@@ -152,8 +151,6 @@ static int GetDeskNumber(char *action, int current_desk)
 	{
 		m = 2;
 	}
-
-
 	if (n > 2)
 	{
 		/* handle limits */
@@ -164,13 +161,25 @@ static int GetDeskNumber(char *action, int current_desk)
 		}
 		else
 		{
-			/*  min > max is nonsense, so swap 'em.  */
+			/* min > max is nonsense, so swap 'em. */
 			min = val[m+1];
 			max = val[m];
 		}
-		if (desk < min)
+		if (is_relative)
 		{
-			/*  Relative move outside of range, wrap around.  */
+			/* Relative moves wrap around.	*/
+			if (desk < min)
+			{
+				desk += (max - min + 1);
+			}
+			else if (desk > max)
+			{
+				desk -= (max - min + 1);
+			}
+		}
+		else if (desk < min)
+		{
+			/* Relative move outside of range, wrap around. */
 			if (val[0] < 0)
 			{
 				desk = max;
@@ -182,7 +191,7 @@ static int GetDeskNumber(char *action, int current_desk)
 		}
 		else if (desk > max)
 		{
-			/*  Relative move outside of range, wrap around.  */
+			/* Move outside of range, truncate. */
 			if (val[0] > 0)
 			{
 				desk = min;
@@ -532,6 +541,11 @@ int HandlePaging(
 
 	do
 	{
+		int rc;
+		Window JunkW;
+		int JunkC;
+		unsigned int JunkM;
+
 		if (FCheckPeekIfEvent(dpy, &e, test_button_event, NULL))
 		{
 			is_timestamp_valid = False;
@@ -539,7 +553,16 @@ int HandlePaging(
 			return 0;
 		}
 		/* get pointer location */
-		fev_get_evpos_or_query(dpy, Scr.Root, pev, &x, &y);
+		rc = FQueryPointer(
+			dpy, Scr.Root, &JunkW, &JunkW, &x, &y, &JunkC, &JunkC,
+			&JunkM);
+		if (rc == False)
+		{
+			/* pointer is on a different screen */
+			x = 0;
+			y = 0;
+		}
+
 		/* check actual pointer location since PanFrames can get buried
 		 * under window being moved or resized - mab */
 		if (x >= edge_thickness &&
@@ -936,21 +959,35 @@ void checkPanFrames(void)
  */
 void raisePanFrames(void)
 {
+	Window windows[4];
+	int n;
+
+	/* Note: make sure the stacking order of the pan frames is not changed
+	 * every time they are raised by using XRestackWindows. */
+	n = 0;
 	if (Scr.PanFrameTop.isMapped)
 	{
-		XRaiseWindow(dpy,Scr.PanFrameTop.win);
+		windows[n++] = Scr.PanFrameTop.win;
 	}
 	if (Scr.PanFrameLeft.isMapped)
 	{
-		XRaiseWindow(dpy,Scr.PanFrameLeft.win);
+		windows[n++] = Scr.PanFrameLeft.win;
 	}
 	if (Scr.PanFrameRight.isMapped)
 	{
-		XRaiseWindow(dpy,Scr.PanFrameRight.win);
+		windows[n++] = Scr.PanFrameRight.win;
 	}
 	if (Scr.PanFrameBottom.isMapped)
 	{
-		XRaiseWindow(dpy,Scr.PanFrameBottom.win);
+		windows[n++] = Scr.PanFrameBottom.win;
+	}
+	if (n > 0)
+	{
+		XRaiseWindow(dpy, windows[0]);
+		if (n > 1)
+		{
+			XRestackWindows(dpy, windows, n);
+		}
 	}
 
 	return;
@@ -1079,10 +1116,11 @@ void MoveViewport(int newx, int newy, Bool grab)
 	if (deltax || deltay)
 	{
 		BroadcastPacket(
-			M_NEW_PAGE, 7, Scr.Vx, Scr.Vy, Scr.CurrentDesk,
-			Scr.MyDisplayWidth, Scr.MyDisplayHeight,
-			(int)(Scr.VxMax / Scr.MyDisplayWidth) + 1,
-			(int)(Scr.VyMax / Scr.MyDisplayHeight) + 1);
+			M_NEW_PAGE, 7, (long)Scr.Vx, (long)Scr.Vy,
+			(long)Scr.CurrentDesk, (long)Scr.MyDisplayWidth,
+			(long)Scr.MyDisplayHeight,
+			(long)((Scr.VxMax / Scr.MyDisplayWidth) + 1),
+			(long)((Scr.VyMax / Scr.MyDisplayHeight) + 1));
 
 		/*
 		 * RBW - 11/13/1998      - new:  chase the chain
@@ -1249,14 +1287,14 @@ void goto_desk(int desk)
 		Scr.CurrentDesk = desk;
 		MapDesk(desk, True);
 		focus_grab_buttons_all();
-		BroadcastPacket(M_NEW_DESK, 1, Scr.CurrentDesk);
+		BroadcastPacket(M_NEW_DESK, 1, (long)Scr.CurrentDesk);
 		/* FIXME: domivogt (22-Apr-2000): Fake a 'restack' for sticky
 		 * window upon desk change.  This is a workaround for a
 		 * problem in FvwmPager: The pager has a separate 'root'
 		 * window for each desk.  If the active desk changes, the
 		 * pager destroys sticky mini windows and creates new ones in
 		 * the other desktop 'root'.  But the pager can't know where to
-		 * stack them.  So we have to tell it ecplicitly where they
+		 * stack them.  So we have to tell it explicitly where they
 		 * go :-( This should be fixed in the pager, but right now the
 		 * pager doesn't maintain the stacking order. */
 		BroadcastRestackAllWindows();
@@ -1909,10 +1947,11 @@ void CMD_DesktopSize(F_CMD_ARGS)
 	Scr.VyMax = (val[1] <= 0) ?
 		0: val[1]*Scr.MyDisplayHeight-Scr.MyDisplayHeight;
 	BroadcastPacket(
-		M_NEW_PAGE, 7, Scr.Vx, Scr.Vy, Scr.CurrentDesk,
-		Scr.MyDisplayWidth, Scr.MyDisplayHeight,
-		(int)(Scr.VxMax / Scr.MyDisplayWidth) + 1,
-		(int)(Scr.VyMax / Scr.MyDisplayHeight) + 1);
+		M_NEW_PAGE, 7, (long)Scr.Vx, (long)Scr.Vy,
+		(long)Scr.CurrentDesk, (long)Scr.MyDisplayWidth,
+		(long)Scr.MyDisplayHeight,
+		(long)((Scr.VxMax / Scr.MyDisplayWidth) + 1),
+		(long)((Scr.VyMax / Scr.MyDisplayHeight) + 1));
 
 	checkPanFrames();
 	/* update GNOME pager */
@@ -1986,7 +2025,7 @@ void CMD_GotoDeskAndPage(F_CMD_ARGS)
 		Scr.CurrentDesk = val[0];
 		MapDesk(val[0], True);
 		focus_grab_buttons_all();
-		BroadcastPacket(M_NEW_DESK, 1, Scr.CurrentDesk);
+		BroadcastPacket(M_NEW_DESK, 1, (long)Scr.CurrentDesk);
 		/* FIXME: domivogt (22-Apr-2000): Fake a 'restack' for sticky
 		 * window upon desk change.  This is a workaround for a
 		 * problem in FvwmPager: The pager has a separate 'root'
@@ -2000,7 +2039,7 @@ void CMD_GotoDeskAndPage(F_CMD_ARGS)
 	}
 	else
 	{
-		BroadcastPacket(M_NEW_DESK, 1, Scr.CurrentDesk);
+		BroadcastPacket(M_NEW_DESK, 1, (long)Scr.CurrentDesk);
 	}
 	EWMH_SetCurrentDesktop();
 	GNOME_SetCurrentDesk();
