@@ -58,6 +58,7 @@ static char *CopyToken(
 	char *out_delim)
 {
 	int len = 0;
+	char *t;
 
 	while (*src != 0 && !(isspace((unsigned char)*src) ||
 			      (snum && strchr(spaces, *src)) ||
@@ -95,8 +96,10 @@ static char *CopyToken(
 		else
 		{
 			if ((*src == '\\' && *(src+1) != 0))
+			{
 				/* Skip over backslashes */
 				src++;
+			}
 			if (len < MAX_TOKEN_LENGTH - 1)
 			{
 				len++;
@@ -114,7 +117,16 @@ static char *CopyToken(
 		*out_delim = *src;
 	}
 	*dest = 0;
-	if (*src != 0)
+	t = SkipSpaces(src, spaces, snum);
+	if (*t != 0 && dnum && strchr(delims, *t) != NULL)
+	{
+		if (out_delim)
+		{
+			*out_delim = *t;
+		}
+		src = t + 1;
+	}
+	else if (*src != 0)
 	{
 		src++;
 	}
@@ -549,7 +561,7 @@ char *GetModuleResource(char *indata, char **resource, char *module_name)
  * ret_suffixnum (0 = no suffix, 1 = first suffix in suffixlist ...).
  *
  */
-int _get_suffixed_integer_arguments(
+static int _get_suffixed_integer_arguments(
 	char *action, char **ret_action, int *retvals, int num,
 	char *suffixlist, int *ret_suffixnum, char *parsestring)
 {
