@@ -71,59 +71,7 @@ static void show_error(const char *msg, ...)
 	va_end(args);
 }
 
-static void change_button_title(button_info *b, const char *text)
-{
-	if (!(b->flags & b_Title))
-	{
-		show_error("Cannot create a title, only change one\n");
-		return;
-	}
-	if (text == NULL)
-	{
-		show_error("No title to change specified, unsupported\n");
-		return;
-	}
-	free(b->title);
-	CopyString(&b->title, text);
-	return;
-}
 
-static void change_button_icon(button_info *b, const char *file)
-{
-	FvwmPicture *new_icon;
-
-	if (!(b->flags & b_Icon))
-	{
-		show_error("Cannot create an icon, only change one\n");
-		return;
-	}
-	if (file == NULL)
-	{
-		show_error("No icon to change specified, unsupported\n");
-		return;
-	}
-	if (LoadIconFile(file, &new_icon, b->colorset) == 0)
-	{
-		show_error("Cannot load icon %s\n", file);
-		return;
-	}
-	free(b->icon_file);
-	PDestroyFvwmPicture(Dpy, b->icon);
-	DestroyIconWindow(b);
-	b->icon = new_icon;
-	CopyString(&b->icon_file, file);
-	CreateIconWindow(b);
-	if (b->flags&b_IconAlpha)
-	{
-		RedrawButton(b, DRAW_FORCE, NULL);
-	}
-	else
-	{
-		ConfigureIconWindow(b, NULL);
-		XMapWindow(Dpy, b->IconWin);
-	}
-	return;
-}
 
 #if 0
 /* to be used in module_expand_action */
@@ -147,8 +95,8 @@ static char *expand_button_vars(const button_info *b, const char *line)
 	char *expanded_line;
 
 	/* there should be a function evaluating fore/back from colorset */
-	char *fore = (b->flags & b_Fore) ? b->fore : "black";
-	char *back = (b->flags & b_Back) ? b->back : "gray";
+	char *fore = (b->flags.b_Fore) ? b->fore : "black";
+	char *back = (b->flags.b_Back) ? b->back : "gray";
 
 	get_button_root_geometry(&r, b);
 	expanded_line = module_expand_action(
@@ -222,7 +170,7 @@ static button_info *parse_button_id(char **line)
 		/* find the button */
 		while (NextButton(&ub, &b, &i, 0))
 		{
-			if ((b->flags & b_Id) && StrEquals(b->id, s))
+			if (b->flags.b_Id && StrEquals(b->id, s))
 			{
 				found = True;
 				break;
@@ -250,9 +198,10 @@ static char *actions[] =
 {
 	"Silent", "ChangeButton", "ExpandButtonVars", NULL
 };
+
 static char *button_options[] =
 {
-	"Title", "Icon", NULL
+	"Title", "Icon", "ActiveTitle", "ActiveIcon", "PressTitle", "PressIcon", NULL
 };
 
 void parse_message_line(char *line)
@@ -288,11 +237,18 @@ void parse_message_line(char *line)
 	{
 	case 1:
 		/* ChangeButton */
+		/* The dimensions of individual buttons (& the overall size of the
+		 * FvwmButtons window) is based on the initial configuration for the
+		 * module. In some configurations, dynamically adding/changing a
+		 * title/icon may mean it no longer fits on a button. Currently, there
+		 * are no checks for this occurance.
+		 */
 		while (rest && rest[0] != '\0')
 		{
 			char *option_pair;
 			int option;
 			char *value0, *value;
+			FvwmPicture *icon;
 
 			/* parse option and value and give diagnostics */
 			rest = GetQuotedString(
@@ -301,10 +257,8 @@ void parse_message_line(char *line)
 			{
 				rest++;
 			}
-	                if (!option_pair)
-        	        {
-                	        continue;
-	                }
+			if (!option_pair)
+				continue;
 
 			option = GetTokenIndex(
 				option_pair, button_options, -1, &value0);
@@ -320,26 +274,84 @@ void parse_message_line(char *line)
 			GetNextToken(value0, &value);
 			free(option_pair);
 
+			if (value == NULL)
+			{
+				show_error("No title/icon to change specified.\n");
+				continue;
+			}
 			switch (option)
 			{
 			case 0:
 				/* Title */
-				change_button_title(b, value);
+				if (b->flags.b_Title)
+					free(b->title);
+				b->flags.b_Title = 1;
+				CopyString(&b->title, value);
 				break;
-			case 1:
-				/* Icon */
-				change_button_icon(b, value);
+			case 2:
+				/* ActiveTitle */
+				if (b->flags.b_ActiveTitle)
+					free(b->activeTitle);
+				b->flags.b_ActiveTitle = 1;
+				CopyString(&b->activeTitle, value);
+				break;
+			case 4:
+				/* PressTitle */
+				if (b->flags.b_PressTitle)
+					free(b->pressTitle);
+				b->flags.b_PressTitle = 1;
+				CopyString(&b->pressTitle, value);
+				break;
+			default:
+				if (LoadIconFile(value, &icon, b->colorset) == 0)
+					show_error("Cannot load icon \"%s\"\n", value);
+				else
+				{
+				    switch (option)
+				    {
+					case 1: /* Icon */
+						if (b->flags.b_Icon)
+						{
+							free(b->icon_file);
+							PDestroyFvwmPicture(Dpy, b->icon);
+						}
+						b->flags.b_Icon = 1;
+						CopyString(&b->icon_file, value);
+						b->icon = icon;
+						break;
+					case 3: /* ActiveIcon */
+						if (b->flags.b_ActiveIcon)
+						{
+							free(b->active_icon_file);
+							PDestroyFvwmPicture(Dpy, b->activeicon);
+						}
+						b->flags.b_ActiveIcon = 1;
+						CopyString(&b->active_icon_file, value);
+						b->activeicon = icon;
+						break;
+					case 5: /* PressIcon */
+						if (b->flags.b_PressIcon)
+						{
+							free(b->press_icon_file);
+							PDestroyFvwmPicture(Dpy, b->pressicon);
+						}
+						b->flags.b_PressIcon = 1;
+						CopyString(&b->press_icon_file, value);
+						b->pressicon = icon;
+						break;
+					}
+				}
 				break;
 			}
 
 			if (value)
-			{
-				free(value);
-			}
+		    {
+			    free(value);
+		    }
 		}
 
 		RedrawButton(b, DRAW_FORCE, NULL);
-		if (UberButton->c->flags & b_TransBack)
+		if (UberButton->c->flags.b_TransBack)
 		{
 			SetTransparentBackground(UberButton, Width, Height);
 		}
