@@ -1,3 +1,4 @@
+/* -*-c-*- */
 /* This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
@@ -276,7 +277,7 @@ static int
 compare_desk (const window_list_entry **u1, const window_list_entry **u2)
 {
   int r = (*u1)->desk - (*u2)->desk;
-  return ((r != 0) ? r : (int) u1 - (int) u2);
+  return ((r != 0) ? r : (long) u1 - (long) u2);
 }
 
 static int
